# -*- coding: utf-8 -*-
#!/usr/bin/env python

import argparse
import os
import lexYaccAnalyse as ly
import numpy as np
import cv2
import sys

sys.path.insert(0, '../')
import DocCreator


knownImageFormats = [".bmp", ".dib", ".lignpeg", ".lignpg", ".lignp2", ".png", ".webp", ".pbm", ".pgm", ".ppm", ".pxm", ".pnm", ".sr", ".ras", ".tiff", ".tif", ".exr", ".hdr", ".pic"]


def exitError(message):
    print(message)
    print("Exiting ...")
    quit()


"""
Function to get all images in a given directory
in : path to the directory or image name
return : image list
"""

def repertory(link):

    bufferImage = []

    if link.endswith(tuple(knownImageFormats)):
        # Only one image
        bufferImage.append(cv2.imread(link))
        return bufferImage

    # Image directory
    if not(os.path.exists(link)):
        # If given directory does not exist, send an error
        exitError("folder param not exist")

    nbImagesFound = 0
    tmpFolderImage = os.listdir(link)

    for image in tmpFolderImage:
        if image.endswith(tuple(knownImageFormats)):
            image = cv2.imread(os.path.join(link, image))
            bufferImage.append(image)
            nbImagesFound += 1

    if nbImagesFound < 1:
        exitError("Given image folder didn't contain any known image formats")

    return bufferImage


"""
Function to get all images in an input folder
return : image list
"""
def repertoryIn(list, bufferImage, indiceLign,  lignParam, counterParam):
    
    inputDir = list.tabDegradation[lignParam][counterParam]
    info = "Repertory in : " + inputDir
    print(info)

    if not(os.path.exists(inputDir)):
        # If given directory does not exist, send an error
        exitError("folder not exist")

    counterParam += 1
    tmpFolderImage = os.listdir(inputDir)
    nbImagesFound = 0

    for image in tmpFolderImage:
        if image.endswith(tuple(knownImageFormats)):
            image = cv2.imread(os.path.join(inputDir, image))
            bufferImage.append(image)
            nbImagesFound += 1

    if nbImagesFound < 1:
        exitError("Given image folder didn't contain any known image formats")

    return counterParam


"""
Function to save images in an output directory
"""
def repertoryOut(list, bufferImage, indiceLign,  lignParam, counterParam):
    
    outputDir = list.tabDegradation[lignParam][counterParam]
    info = "Repertory out : " + outputDir
    print(info)

    if not(os.path.exists(outputDir)):
        # If given directory does not exist, it will be created
        os.makedirs(outputDir)

    counterParam += 1
    x = 0

    for image in bufferImage:
        x = x + 1
        name = "image_" + str(x) + ".png"
        cv2.imwrite(os.path.join(outputDir, name), image)

    return counterParam


"""
Function to apply a degradation to the images of the current buffer
return : image list
Sorti : liste d'image
"""
def functionDegredation(list, bufferImage, indiceLign, lignParam, counterParam):

    indiceLign += 1
    bufferImageTemp = []
    func = list.tabDegradation[lignParam][counterParam]
    info = "Function : " + func
    print(info)
    counterParam += 1
    args = []


    # === Functions that contains a string in their args ===

    if (func == "phantomCharacter"):

        for i in range(int(list.infoLign[lignParam][indiceLign])):
            if (i == 1):
                # When there is only one image
                args.append(str(list.tabDegradation[lignParam][counterParam]))
            else :
                args.append(int(list.tabDegradation[lignParam][counterParam]))
            counterParam += 1

        arg0 = args[0]
        link = args[1]
        argImage = repertory(link)
        numberImageArg = len(argImage)
        counterImageArg = 0

        for image in bufferImage:
            bufferImageTemp.append(DocCreator.phantomCharacter(image, arg0,link))
            counterImageArg += 1
            if (counterImageArg == numberImageArg):
                counterImageArg = 0

        return counterParam, indiceLign, bufferImageTemp


    elif (func == "bleedThrough"):

        for i in range(int(list.infoLign[lignParam][indiceLign])):
            if (i == 0):
                # When there is only one image
                args.append(str(list.tabDegradation[lignParam][counterParam]))
            else:
                args.append(int(list.tabDegradation[lignParam][counterParam]))
            counterParam += 1

        link = args[0]
        args = args[1:]
        argImage = repertory(link)
        numberImageArg = len(argImage)
        counterImageArg = 0

        for image in bufferImage:
            bufferImageTemp.append(DocCreator.bleedThrough(image, argImage[counterImageArg], *args))
            counterImageArg += 1
            if (counterImageArg == numberImageArg):
                counterImageArg = 0

        return counterParam, indiceLign, bufferImageTemp


    elif (func == "holeDegradation"):

        tailleArg = list.infoLign[lignParam][indiceLign]

        if (list.infoLign[lignParam][indiceLign+1].isdigit()):
            tailleArg = int(list.infoLign[lignParam][indiceLign]+list.infoLign[lignParam][indiceLign+1])
        else:
            tailleArg = int(list.infoLign[lignParam][indiceLign])

        for i in range(tailleArg):
            if (i == 0 or i == 7):  # Args number for holeDegradation()
                args.append(str(list.tabDegradation[lignParam][counterParam]))
            elif (i == 6):
                colorTemp = (str(list.tabDegradation[lignParam][counterParam]).split(':'))
                color = []
                color.append(int(colorTemp[0]))
                color.append(int(colorTemp[1]))
                color.append(int(colorTemp[2]))
                args.append(color)
            else :
                args.append(int(list.tabDegradation[lignParam][counterParam]))
            counterParam += 1

        link1 = args[0]
        argImage = repertory(link1)
        numberImageArg = len(argImage)
        counterImageArg = 0

        if (tailleArg > 7):
            link2 = args[7]
            argImage2 = repertory(link2)

        else:
            args.append(None)
            argImage2 = [None,None]
            args.append(0)
            args.append(1000)

        numberImageArg2 = len(argImage2)
        counterImageArg2 = 0

        for image in bufferImage:
            bufferImageTemp.append(DocCreator.holeDegradation(image,argImage[counterImageArg],
                args[1], args[2], args[3], args[4], args[5],args[6],argImage2[counterImageArg2],
                                  args[8], args[9]))
            counterImageArg += 1
            counterImageArg2 += 1
            if (counterImageArg == numberImageArg):
                counterImageArg = 0
            if (counterImageArg2 == numberImageArg2):
                counterImageArg2 = 0
        return counterParam, indiceLign, bufferImageTemp


    # === Functions that does not contain any string in their args

    for i in range(int(list.infoLign[lignParam][indiceLign])):
        args.append(int(list.tabDegradation[lignParam][counterParam]))
        counterParam += 1

    if (func == "grayscaleChars"):
        for image in bufferImage:
            bufferImageTemp.append(DocCreator.grayscaleChars(image, *args))

    if (func == "shadowBinding"):
        for image in bufferImage:
            bufferImageTemp.append(DocCreator.shadowBinding(image, *args))

    if (func == "blurFilter"):
        for image in bufferImage:
            bufferImageTemp.append(DocCreator.blurFilter(image, *args))

    return counterParam, indiceLign, bufferImageTemp



def generation(list, bufferImage, indiceLign, lignParam, counterParam):

    info = "Generation "
    print(info)
    indiceLign += 1
    func = list.tabDegradation[lignParam][counterParam]
    counterParam += 1
    args = []

    """
    arg1 : output folder
    arg2 : image group
    arg3 : string group
    arg4 : path to text files
    """

    functionID = 0
    for i in range(int(list.infoLign[lignParam][indiceLign])):
        if (i == 0):
            backgroundList = (str(list.tabDegradation[lignParam][counterParam]).split(':'))

        elif(i == 1) :
            fontList = (str(list.tabDegradation[lignParam][counterParam]).split(':'))

        elif(i == 2):
            textDirPath = str(list.tabDegradation[lignParam][counterParam])

        else:
            functionID = int(list.tabDegradation[lignParam][counterParam])
        counterParam += 1

    inputDir = 'tempGeneration'

    if (textDirPath == "LoremIpsum"):
        params = {
            'outputFolderPath' : inputDir,
            'backgroundList' : backgroundList,
            'fontList' : fontList,
        }

    else :
        params = {
            'outputFolderPath' : inputDir,
            'backgroundList' : backgroundList,
            'fontList' : fontList,
            'txtDirPath' : textDirPath,
        }

    if (functionID == 0):
        DocCreator.create(params)
    if (functionID == 1):
        DocCreator.createAllFontsOneBackground(params)
    if (functionID == 2):
        DocCreator.createOneFontAllBackgrounds(params)
    if (functionID == 3):
        DocCreator.createAllFontsAllBackgrouds(params)

    tmpFolderImage = os.listdir(inputDir)
    nbImagesFound = 0

    for image in tmpFolderImage:
        if image.endswith(tuple(knownImageFormats)):
            image = cv2.imread(os.path.join(inputDir, image))
            bufferImage.append(image)
            nbImagesFound += 1

    deleteRep = "rm -r "+ inputDir
    os.system(deleteRep)
    return counterParam, indiceLign, bufferImage



# === ARGPARSE PART ===

# Initialization
parser = argparse.ArgumentParser(description="DocCreator CLI")

# Degradation chain, must have argument
parser.add_argument("expression", metavar="expression", help="expression of degradation, see the doc")

argsParser = parser.parse_args()


# === LEXER PART ===

# Lexer construction
lexer = ly.lexYaccAnalyse()

# Lex part construction
lexer.buildLexer()

# Yacc part construction
lexer.buildParser()

if('=>' in argsParser.expression):
    expression = argsParser.expression.replace('\n','').replace('\t','')
    error = lexer.testParser(argsParser.expression)
else:
    configFile = open(argsParser.expression, "r")
    expression = ""
    for lign in configFile:
        expression += lign+"&&"
    expression = expression[:len(expression)-2]
    expression = expression.replace('\n','').replace('\t','')
    error = lexer.testParser(expression)

if (str(error) == '0'):
    print("expression Ok")
else:
    result = "error : " + str(error)
    exitError(result)

list = lexer.getList()
print(list.infoLign)

for lignParam in range(list.lign + 1):  # Each line is separated with an "and"

    indiceLign = 0
    counterParam = 0
    bufferImage = []

    while indiceLign < len(list.infoLign[lignParam]):

        # Image generation
        if list.infoLign[lignParam][indiceLign] == 'g':
            counterParam, indiceLign, bufferImage = generation(list, bufferImage, indiceLign,
                                                  lignParam, counterParam)

        # Degradation
        if list.infoLign[lignParam][indiceLign] == 'f':
            counterParam, indiceLign, bufferImage = functionDegredation(list, bufferImage, indiceLign,  lignParam, counterParam)

        # Folder management
        if list.infoLign[lignParam][indiceLign] == 'r':
            if (indiceLign == 0):
                counterParam = repertoryIn(list, bufferImage, indiceLign, lignParam, counterParam)
            else:
                counterParam = repertoryOut(list, bufferImage, indiceLign, lignParam, counterParam)
                
        indiceLign = indiceLign + 1
