# DocCreatorPackage

Ce dossier contient tous les fichiers nécessaires à la génération du package _pip_ de la version Python de DocCreator.

Dans la version actuelle, notre DocCreatorPackage permet d'utiliser le wrapper Python de _GrayscaleCharsDegradationModel_ (fichiers **_GrayscaleCharsDM.so** et **GrayscaleCharsDM.py**). Le fichier **cli.py** est également inclut dans le paquet mais ne peut pas (encore) être utilisé comme un exécutable.


## Installation du paquet :

Le package est actuellement en phase de test, la commande permettant de l'installer est donc :

```bash
python -m pip install --index-url https://test.pypi.org/simple/ DocCreatorPackage
```

La bibliothèque peut être utilisée dans un fichier Python avec :
```python
import DocCreator
```
L'interface en ligne de commande peut être utilisée avec :
```bash
python -m DocCreator.cli [arguments]
```


## Utilisation :

Deux méthodes :

1) Importer avec :
```python
from DocCreator.GrayscaleCharsDM import *
```
puis créer l'objet correspondant, exemple :
```python
gs = GrayscaleCharsDM("image.png")
```

Les trois fonctions de dégradations alors disponibles sont :
- Dégrader l'image, les arguments 'level' (niveau), 'I' (points indépendants), 'O' (points qui chevauchent des caractères) et 'D' (points discontinus) sont optionnels, et utiliseront leur valeur par défaut si non spécifiés (attention, la somme de I, O et D doit toujours être égale à 100) :
```python
gs.degradate("output.png", level=1, I=33, O=33, D=34)
```
- Dégrader l'image par niveaux :
```python
gs.degradateByLevel("output-lv2.png", 2)
```
- Récupérer l'image en noir et blanc :
```python
gs.getImageGray("output-gray.png")
```

Enfin on supprime l'objet avec :
```python
del gs
```

2) Importer avec :
```python
from DocCreator import GrayscaleCharsDM
```
Le fonctionnement est ensuite le même que pour la méthode précédente, mais toutes les fonctions doivent être appelées avec le préfixe "GrayscaleCharsDM" telles que :
```python
gs = GrayscaleCharsDM.GrayscaleCharsDM("image.png")
```


## Mettre à jour les fichiers du paquet :

Vérifier que _setuptools wheel_ et _twine_ sont bien installés :
```
python -m pip install --user --upgrade setuptools wheel twine
```

Après avoir modifié les fichiers à mettre à jour, il est nécessaire d'incrémenter le numéro de version de **setup.py**.
Il faut ensuite regénérer le contenu du dossier **/dist** :
```
rm -rf dist
python setup.py sdist bdist_wheel
```

Enfin pour uploader le nouveau paquet :
```
python -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*
```

Des identifiants de connexion seront alors demandés, de manière temporaire, ceux-ci ont été définis par :
    - Login :        pdp-bfhl
    - Mot de passe : Lbgfahql


## Ajouter de nouveaux fichiers :

Pour ajouter de nouveaux fichiers au package, deux procédures :

- S'il s'agit d'un fichier Python .py :
  Il suffit de l'ajouter dans le dossier /DocCreatorPackage/DocCreator avec les autres

- S'il s'agit d'un .so :
  Il est nécessaire de l'ajouter dans la ligne __package_data__ de **setup.py**, puis de l'ajouter également dans le dossier /DocCreatorPackage/DocCreator

Ensuite il faut mettre à jour le paquet suivant la procédure de mise à jour des fichiers décrite précédemment.
