#include <iomanager/fontfilemanager.h>
#include <QDebug>
#include <Utils/convertor.h>
#include <RandomDocument/DocumentGeneration.hpp>

using namespace std;

DocumentGeneration::DocumentGeneration() {
    _imageNumber = 0;
}

void DocumentGeneration::generate() {
  if (!QDir(_outputFolder).exists()){
      QDir().mkdir(_outputFolder);
  }

  QString txtTextDir = _txtDir;
  if (txtTextDir.length() != 0 && !txtTextDir.isEmpty()) {
    QDir folder(txtTextDir);

    QStringList filters;
    filters << QStringLiteral("*.txt");
    folder.setNameFilters(filters);

    QStringList listFileNames = folder.entryList();

    int per = 100 / listFileNames.size();
    if (!listFileNames.empty())
      for (const QString &file : listFileNames) {
        QString filePath = folder.absoluteFilePath(file);
        createDocument(filePath);
        per += 100 / listFileNames.size();
      }
  }
  // No texts, we will generate a Lorem Ipsum
  else {
    createDocument(NULL);
  }
}

void DocumentGeneration::saveImage(Doc::Document *document, const QString &file) {
  QImage out;

  if (_Char_Format < 7 && _Char_Format >= 1) {
    out = _bg.convertToFormat(_Char_Format);
  } else {
    out = _bg.convertToFormat(QImage::Format_ARGB32);
  }

  if (document != nullptr && !document->getPages().empty())
    for (Doc::Page *page : document->getPages()) {

      if (!page->getTextBlocks().empty())
        for (Doc::DocTextBlock *tb : page->getTextBlocks()) {
          int tb_posX = tb->x();
          int tb_posY = tb->y(); // adjust the text block position

          int counter_lines = 0;
          int checker_width = 0;

          if (!tb->getParagraphs().empty()) {
            tb_posY += tb->getParagraphs().at(0)->lineSpacing() /
                       3; // adjust the text block position

            for (Doc::DocParagraph *p : tb->getParagraphs()) {
              int wo = 0, ho = 0;

              if (!p->getStrings().empty()) {
                for (Doc::DocString *s : p->getStrings()) {
                  if (!s->getCharacters().empty())
                    for (Doc::DocCharacter *ch : s->getCharacters()) {
                      Models::Character *character =
                        _font->getCharacter(ch->getDisplay());
                      const QString chDisplay = ch->getDisplay();
                      if (character == nullptr ||
                          chDisplay == QStringLiteral(" ") ||
                          chDisplay.isEmpty())
                        continue;

                      const Models::CharacterData *charData =
                        character->getCharacterData(ch->getId());
                      const QImage charImage = charData->getImage();

                      wo = charImage.width();
                      ho = charImage.height();
                      if (wo > 0 && ho > 0)
                        break;
                    }

                  if (wo > 0 && ho > 0)
                    break;
                }

                for (Doc::DocString *s : p->getStrings()) {

                  if (!s->getCharacters().empty())
                    for (Doc::DocCharacter *c : s->getCharacters()) {
                      int posX = c->x();

                      if (checker_width > c->x())
                        ++counter_lines; // count the number of lines

                      int posY = c->y() - counter_lines * p->lineSpacing() /
                                            2; // adjust the character position

                      const QString cDisplay = c->getDisplay();
                      Models::Character *character =
                        _font->getCharacter(cDisplay);

                      if (character != nullptr && !cDisplay.isEmpty()) {
                        const Models::CharacterData *charData =
                          character->getCharacterData(c->getId());
                        const QImage charImage = charData->getImage();

                        if (ho > 0)
                          posY += ho - charImage.height();
                        posY += (1 - character->getBaseLine() / (float)100) *
                                charImage.height();

                        /** DEBUG **
                            if (id ==2) qDebug() << c->getDisplay() << " char w " << charImage->width() << " h " << charImage->height() <<
                            " char x y "  << posX << " " << posY <<
                            " tb x y " << tb_posX << " " << tb_posY <<
                            " out w h " << out.width() << " " << out.height() <<
                            " total x y " << (charImage->width() + posX + tb_posX) << " " << (charImage->height() + posY + tb_posY);
                        */

                        //B:TODO:OPTIM: remove checks from inside of loops !
                        //B:TODO:OPTIM: do not use pixel()/setPixel() !
                        const int w = charImage.width();
                        const int h = charImage.height();
                        for (int y = 0; y < h; ++y) {
                          for (int x = 0; x < w; ++x) {
                            const int alpha = qAlpha(charImage.pixel(x, y));
                            if ((x + tb_posX + posX) < out.width() &&
                                (y + tb_posY + posY) < out.height()) {
                              if (alpha != 0) {
                                out.setPixel(x + tb_posX + posX,
                                             y + tb_posY + posY,
                                             charImage.pixel(x, y));
                              }
                            }
                          }
                        }
                      }
                      checker_width = c->x();
                    }
                }
              }
            }
          }
        }
      QString imgToSave = _outputFolder + "/"+ file + ".png";
      out.save(imgToSave);
    }

}


void DocumentGeneration::loadFont() {
  //_font = Context::FontContext::instance()->getFont(_fontName);

  const Models::CharacterMap charsMap = _font->getCharacters();

  //QMap<QString, Character*>
  Models::CharacterMap::const_iterator it = charsMap.constBegin();

  int counter = 0;
  const Models::CharacterMap::const_iterator itEnd = charsMap.constEnd();
  for (; it != itEnd; ++it) {

    Models::Character *ch = it.value();
    Models::CharacterDataList charsDataList = ch->getAllCharacterData();

    for (Models::CharacterData *charData : charsDataList) {

      if (charData != nullptr) {
        const QImage charImage = charData->getImage();
        if (!charImage.isNull()) {
          _word_height += charImage.height();
          _Char_Format = charImage.format();

          ++counter;
        }
      }
    }
  }

  _word_height = _word_height / counter;
  _word_height = round(1.5 * _word_height);
}

void DocumentGeneration::setupPageParameters() {
  _columnSpacing = 10;
  _word_height = 0;


  _width = _bg.width();
  _height = _bg.height();

  loadFont();

  _nbColumns = 1; // text block layout
  _columnsWidth = (_width - _pageMarginLeft - _pageMarginRight -
                   _columnSpacing * (_nbColumns - 1)) /
                  _nbColumns;
  _columnsHeight = (_height - _pageMarginTop - _pageMarginBottom);
}

Doc::Document * DocumentGeneration::createADocument() {
  setupPageParameters();
  auto document = new Doc::Document();
  auto style = new Doc::DocStyle(_fontName);
  document->addStyle(style);
  return document;
}

Doc::Page * DocumentGeneration::createAPage(Doc::Document *document) {
  auto page = new Doc::Page(document);
  page->setBackgroundFileName(_bgName);
  document->add(page);
  document->setPageWidth(_width);
  document->setPageHeight(_height);
  document->setCurrentPage(page);
  return page;
}

Doc::DocTextBlock * DocumentGeneration::createATextBox(Doc::Document *document,
                                               int x,
                                               int y)
{
  auto tb = new Doc::DocTextBlock(document);
  tb->setMarginTop(0);
  tb->setMarginBottom(0);
  tb->setMarginLeft(0);
  tb->setMarginRight(0);

  tb->setHeight(_columnsHeight);
  tb->setWidth(_columnsWidth);
  tb->setX(x);
  tb->setY(y);

  return tb;
}

QImage DocumentGeneration::getCharacterImage(Models::Character *ch) {
  const Models::CharacterData *charData = ch->getRandomCharacterData();
  const QImage charImage = charData->getImage();
  return charImage;
}

void DocumentGeneration::bindTextForATextBox(QStringList &textLines, Doc::DocTextBlock *tb) {
  cv::Point char_pos(0, 0);
  bool isFullBlock = false;

  while (!textLines.isEmpty() && !isFullBlock) {

    if ((char_pos.y + _word_height) > _columnsHeight) {
      isFullBlock = true;
      break;
    } // if the block is full -> out

    QString &line = textLines.first(); // get first line

    auto p = new Doc::DocParagraph(tb->getDocument()); // create a new paragraph
    p->setLineSpacing(_lineSpacing);
    tb->add(p); // add the current paragraph to the current textblock of page.

    while (!line.isEmpty()) {

      QChar c = line.at(0); // Get the current character
      Models::Character *character = _font->getCharacter(QString(c));

      if (character != nullptr) {

        const int char_ID = character->getRandomCharacterData()->getId();
        const QImage charImage = character->getCharacterData(char_ID)->getImage();

        // check line return
        if ((char_pos.x + charImage.width()) > _columnsWidth) {
            char_pos.x = 0;
            char_pos.y += _word_height + _lineSpacing; // line return
        }

        if ((char_pos.y + _word_height) > _columnsHeight) {
          isFullBlock = true;
          break;
        } // IF the block is full -> out

        Doc::DocCharacter *docChar =
          new Doc::DocCharacter(QString(c), char_ID, tb->getDocument());
        docChar->setX(char_pos.x);
        docChar->setY(char_pos.y);
        docChar->setWidth(charImage.width());
        docChar->setHeight(charImage.height());
        p->add(docChar);

        char_pos.x += charImage.width(); // update next position
      }
      line.remove(0, 1); // remove the current character
    }

    if (isFullBlock)
      break;                 // IF the block is full -> out
                             // else
    textLines.removeFirst(); // remove the first line

    // line return
    char_pos.x = 0;
    char_pos.y += _word_height + _lineSpacing;

    if (textLines.isEmpty())
      isFullBlock = true; // Text line is empty
  }
}

Doc::Page * DocumentGeneration::bindTextForAPage(Doc::Document *document,
                                         QStringList &textlines)
{

  Doc::Page *page = createAPage(document);

  int tb_x = _pageMarginLeft;
  int tb_y = _pageMarginTop;

  for (int i = 0; i < _nbColumns; ++i) { // add textbox
    if (textlines.isEmpty())
      break; // check text lines firstly

    Doc::DocTextBlock *tb = createATextBox(document, tb_x, tb_y);
    bindTextForATextBox(textlines, tb);

    page->add(tb);
    page->setCurrentBlock(tb);

    tb_x += _columnsWidth + _columnSpacing;
  }

  return page;
}

QStringList DocumentGeneration::readTextFile(const QString &filePath) {
  std::string line;
  std::ifstream file(filePath.toStdString().c_str());
  QStringList textLines;
  if (file.is_open()) {
    while (file.good()) {
      getline(file, line);
      if (!line.empty())
        textLines.push_back(QString::fromStdString(line));
    }
  }
  file.close();
  return textLines;
}

void DocumentGeneration::createDocument(const QString &filePath) {
  Doc::Document *document = createADocument();
  QStringList textLines;
  if (filePath == NULL) {
    textLines.append(Lipsum);
  } else {
    textLines = readTextFile(filePath);
  }

  int pageCounter = 0;
  _imageNumber++;
  while (!textLines.isEmpty()) {
    Doc::Page *page = bindTextForAPage(document, textLines);

    const QString imageFilename =
          "Document" + QString::number(_imageNumber)+"_p" + QString::number(pageCounter);

    saveImage(document, imageFilename);

    ++pageCounter;
    document->remove(page);
    delete page;
  }
  delete document;

}


const QString DocumentGeneration::Lipsum = QStringLiteral(
  "Lorem ipsum dolor sit amet, consectetur adipiscing elit. In semper massa "
  "sapien, id pulvinar justo elementum tincidunt. Cras vel semper enim. "
  "Integer tortor nisl, maximus eget dolor eu, pulvinar iaculis metus. Nulla "
  "malesuada convallis nisi, sed dictum mauris gravida quis. Sed a dui nisl. "
  "Nullam gravida quam quis dapibus vehicula. Proin in sapien orci. Vivamus et "
  "placerat nibh. Aliquam erat volutpat. Cum sociis natoque penatibus et "
  "magnis dis parturient montes, nascetur ridiculus mus. \n Suspendisse "
  "rhoncus nibh sed dui dapibus efficitur. Morbi tempor egestas condimentum. "
  "Nam sit amet dolor lacus. Cras a lectus pulvinar, tempor neque eleifend, "
  "pharetra felis. Proin gravida ex justo, nec placerat mauris sollicitudin "
  "in. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per "
  "inceptos himenaeos. Cras non tellus in sem eleifend rutrum vitae et ipsum. "
  "Nulla in nisi sollicitudin, hendrerit urna imperdiet, rutrum purus. Sed "
  "fermentum felis bibendum lorem porta sodales. Class aptent taciti sociosqu "
  "ad litora torquent per conubia nostra, per inceptos himenaeos. Aliquam "
  "elementum arcu in sapien sollicitudin, vitae lobortis justo "
  "venenatis.\nCurabitur sit amet est ac libero varius consequat. Sed auctor "
  "lobortis pharetra. Nam feugiat, eros at facilisis rhoncus, lectus ligula "
  "blandit est, sit amet volutpat diam massa eget massa. Nunc mi nulla, "
  "accumsan sit amet erat in, dapibus porta est. Aliquam aliquam malesuada "
  "venenatis. Donec elementum nibh vitae elit vehicula placerat. Fusce mattis, "
  "tellus vitae iaculis dignissim, quam nunc posuere neque, at porta leo odio "
  "tincidunt erat. Fusce eleifend malesuada elit at pellentesque. Nam feugiat "
  "felis quis turpis posuere, in varius erat pharetra. In aliquam fringilla "
  "lectus, id ultricies lacus. Interdum et malesuada fames ac ante ipsum "
  "primis in faucibus.\nSed porttitor nec dolor id pellentesque. Donec "
  "faucibus, ex in porta consequat, felis tortor volutpat quam, in egestas "
  "quam tortor et lacus. Maecenas rutrum porttitor vehicula. Interdum et "
  "malesuada fames ac ante ipsum primis in faucibus. Donec sed pretium odio. "
  "Nullam odio nunc, hendrerit quis pharetra nec, porttitor vitae turpis. ");