#include <string>
#include <QDir>

#include "context/backgroundcontext.h"
#include "context/documentcontext.h"
#include "context/fontcontext.h"
#include "iomanager/documentfilemanager.h"
#include "models/character.h"
#include "models/characterdata.h"
#include "models/doc/doctextblock.h"
#include "models/doc/document.h"
#include "models/doc/page.h"
#include "models/font.h"

#include <cassert>
#include <cmath>
#include <fstream>
#include <vector>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>

namespace Doc {
class DocTextBlock;
class Document;
class Page;
}
namespace Models {
class Character;
class Font;
}

class DocumentGeneration
{
public:
  DocumentGeneration();

  void generate();



private:
  void saveImage(Doc::Document *document, const QString &file);
  void createDocument(const QString &filePath);

  Doc::Document *createADocument();
  Doc::Page *createAPage(Doc::Document *document);
  Doc::DocTextBlock *createATextBox(Doc::Document *document, int x, int y);
  QImage getCharacterImage(Models::Character *ch);
  void bindTextForATextBox(QStringList &textLines, Doc::DocTextBlock *tb);
  Doc::Page *bindTextForAPage(Doc::Document *document, QStringList &TEXT_lines);
  QStringList readTextFile(const QString &filePath);

  void loadFont();
  void setupPageParameters();

public:
  Models::Font * _font;
  QString _txtDir;
  QString _outputFolder;
  QString _bgName;
  QImage _bg;
  int _pageMarginTop;
  int _pageMarginLeft;
  int _pageMarginRight;
  int _pageMarginBottom;
  int _lineSpacing;

private:
  int _imageNumber;
  QString _fontName;
  float _word_height;

  int _columnSpacing;
  int _width;
  int _height;
  QImage::Format _Char_Format;

  int _nbColumns;
  int _columnsWidth;
  int _columnsHeight;

  const static QString Lipsum;
};