# -*- coding: utf-8 -*-

import argparse
import os
import numpy as np
import cv2
import sys

sys.path.insert(0, 'wrapper/python')
import testDegradationWrapper as TestDeg

sys.path.insert(0, '../wrapper/python')
import DocCreator



cliPath = 'cli.py'
cliExe = "python " + cliPath + " "

os.chdir(os.path.join('..', 'wrapper', 'python', 'cli'))

refImgDir = os.path.join('..', '..', '..', 'tests', 'data', 'images', 'refImages', '')
refImg1 = os.path.join(refImgDir, 'p00.png')
refImg2 = os.path.join(refImgDir, 'p01.png')

otherImgDir = os.path.join('..', '..', '..', 'tests', 'data', 'images', 'other', '')
patternImg = os.path.join(otherImgDir, 'pattern1.png')

outputDirCLI = "testCLIcli"
outputDirWrapper = "testCLIwrapper"



def exitError(degradation, func):
    print("Error testing " + degradation + " while calling " + func + " function")
    sys.exit(1)



def testShadowBinding():

    TestDeg.manageDir(outputDirCLI)
    TestDeg.manageDir(outputDirWrapper)
    degradation = "shadowbinding"

    # Testing shadowBinding() with border type 3 (left) and distance (size in pixel) at 500
    arg1 = 3
    arg2 = 500

    params = "\"r:" + refImgDir + "=>shadowBinding(" + str(arg1) + "," + str(arg2) + ")=>r:" + outputDirCLI + "\""

    # CLI
    try:
        os.system(cliExe + params)
    except:
        exitError(degradation, "cli")

    # Wrapper
    try:
        img = cv2.imread(refImg1)
        outImg = DocCreator.shadowBinding(img, arg1, arg2)
        TestDeg.saveImg(degradation, 1, outputDirWrapper, outImg)

        img = cv2.imread(refImg2)
        outImg = DocCreator.shadowBinding(img, arg1, arg2)
        TestDeg.saveImg(degradation, 2, outputDirWrapper, outImg)
    except:
        exitError(degradation, "wrapper")

    TestDeg.compareDir(degradation, outputDirCLI, outputDirWrapper)


def testHoleDegradation():

    TestDeg.manageDir(outputDirCLI)
    TestDeg.manageDir(outputDirWrapper)
    degradation = "holedegradation"

    # Testing holeDegradation() with holeType 1
    arg1 = 1

    params = "\"r:" + refImgDir + "=>holeDegradation("+ patternImg + ",50,50,0," + str(arg1) + ",1,128:128:128)=>r:" + outputDirCLI + "\""

    # CLI
    try:
        os.system(cliExe + params)
    except:
        exitError(degradation, "cli")

    # Wrapper
    try:
        img = cv2.imread(refImg1)
        img2 = cv2.imread(patternImg)
        outImg = DocCreator.holeDegradation(img, img2, 50, 50, 0, arg1, 1, [128, 128, 128])
        TestDeg.saveImg(degradation, 1, outputDirWrapper, outImg)

        img = cv2.imread(refImg2)
        outImg = DocCreator.holeDegradation(img, img2, 50, 50, 0, arg1, 1, [128, 128, 128])
        TestDeg.saveImg(degradation, 2, outputDirWrapper, outImg)
    except:
        exitError(degradation, "wrapper")

    TestDeg.compareDir(degradation, outputDirCLI, outputDirWrapper)


def testBleedThrough():

    TestDeg.manageDir(outputDirCLI)
    TestDeg.manageDir(outputDirWrapper)
    degradation = "bleedthrough"

    # Testing bleddThrough() with nbIter at 50
    arg1 = 50

    params = "\"r:" + refImgDir + "=>bleedThrough("+ refImg2 + "," + str(arg1) + ")=>r:" + outputDirCLI + "\""

    # CLI
    try:
        os.system(cliExe + params)
    except:
        exitError(degradation, "cli")

    # Wrapper
    try:
        img = cv2.imread(refImg1)
        img2 = cv2.imread(refImg2)
        outImg = DocCreator.bleedThrough(img, img2, arg1)
        TestDeg.saveImg(degradation, 1, outputDirWrapper, outImg)

        img = cv2.imread(refImg2)
        outImg = DocCreator.bleedThrough(img, img2, arg1)
        TestDeg.saveImg(degradation, 2, outputDirWrapper, outImg)
    except:
        exitError(degradation, "wrapper")

    TestDeg.compareDir(degradation, outputDirCLI, outputDirWrapper)


def testBlurFilter():

    TestDeg.manageDir(outputDirCLI)
    TestDeg.manageDir(outputDirWrapper)
    degradation = "blurfilter"

    # Testing blurFilter() with method : Gaussian (0) and intensity at 11 (medium value)
    arg1 = 0
    arg2 = 11

    params = "\"r:" + refImgDir + "=>blurFilter(" + str(arg1) + "," + str(arg2) + ")=>r:" + outputDirCLI + "\""

    # CLI
    try:
        os.system(cliExe + params)
    except:
        exitError(degradation, "cli")

    # Wrapper
    try:
        img = cv2.imread(refImg1)
        outImg = DocCreator.blurFilter(img, arg1, arg2)
        TestDeg.saveImg(degradation, 1, outputDirWrapper, outImg)

        img = cv2.imread(refImg2)
        outImg = DocCreator.blurFilter(img, arg1, arg2)
        TestDeg.saveImg(degradation, 2, outputDirWrapper, outImg)
    except:
        exitError(degradation, "wrapper")

    TestDeg.compareDir(degradation, outputDirCLI, outputDirWrapper)



def main (degradation):

    if (degradation == 'shadowbinding'):
        testShadowBinding()
    elif (degradation == 'holedegradation'):
        testHoleDegradation()
    elif (degradation == 'bleedthrough'):
        testBleedThrough()
    elif (degradation == 'blurfilter'):
        testBlurFilter()
    else:
        print("Unknown degradation, please use one of the following : ")
        print("shadowbinding, holedegradation, bleedthrough, blurfilter")
        sys.exit(1)


if __name__ == "__main__":
    if (len(sys.argv) < 2):
        print("Error launching tests, you need to specify a degradation to test.")
        sys.exit(1)
    main(sys.argv[1])

