import Degradations
import numpy as np
import random


''' DEGRADATIONS
For each degradations we write a new function to ensure that the shape of the result
is the same as the shape of the input and to make the reshape transparent to the user.
'''

def grayscaleChars(imgIn, level=1, I=33, O=33, D=34):
    imgOutLength = imgIn.shape[0] * imgIn.shape[1] * imgIn.shape[2]
    res = Degradations.grayscaleChars(imgIn, imgOutLength, level, I, O, D)
    return res.reshape(imgIn.shape)

def grayscaleCharsByLevel(imgIn, level):
    imgOutLength = imgIn.shape[0] * imgIn.shape[1] * imgIn.shape[2]
    res = Degradations.grayscaleCharsByLevel(imgIn, imgOutLength, level)
    return res.reshape(imgIn.shape)

def shadowBinding(imgIn, border, distance, intensity=0.5, angle=30):
    imgOutLength = imgIn.shape[0] * imgIn.shape[1] * imgIn.shape[2]
    res = Degradations.shadowBinding(imgIn, imgOutLength, border, distance, intensity, angle)
    return res.reshape(imgIn.shape)

def holeDegradation(imgIn, pattern, xOrigin, yOrigin, size, holetype,
 side, color, img2=None, width=0, intensity=1000):
    imgOutLength = imgIn.shape[0] * imgIn.shape[1] * imgIn.shape[2]
    if (img2 is None):
        '''
        There is some problem when passing a Python None to a C++ function instead
        of NULL so we call the function without the concerned args (width and
        intensity have effects only if we fill the hole with another image)
        '''
        res = Degradations.holeDegradation(imgIn, imgOutLength, pattern, xOrigin,
                yOrigin, size, holetype, side, color)
    else:
        res = Degradations.holeDegradation(imgIn, imgOutLength, pattern, xOrigin,
                yOrigin, size, holetype, side, color, img2, width, intensity)
    return res.reshape(imgIn.shape)

def bleedThrough(imgIn, img2, nbIter, x=0, y=0, nbThreads=-1):
    imgOutLength = imgIn.shape[0] * imgIn.shape[1] * imgIn.shape[2]
    res = Degradations.bleedThrough(imgIn, imgOutLength, img2, nbIter, x, y, nbThreads)
    return res.reshape(imgIn.shape)

def phantomCharacter(imgIn, frequency, phantomPatternsPath):
    imgOutLength = imgIn.shape[0] * imgIn.shape[1] * imgIn.shape[2]
    res = Degradations.phantomCharacter(imgIn, imgOutLength, frequency, phantomPatternsPath)
    return res.reshape(imgIn.shape)

def blurFilter(imgIn, method, intensity):
    imgOutLength = imgIn.shape[0] * imgIn.shape[1] * imgIn.shape[2]
    res = Degradations.blurFilter(imgIn, imgOutLength, method, intensity)
    return res.reshape(imgIn.shape)


