# Wrapper Python

## Installation des dépendances
_Linux_ :
Utilisez le script `install-dependencies.sh` présent à la racine du projet pour installer toutes les dépendances (nécéssite les droits sudo).

_MacOS_ :
Installer Qt via l'installeur officiel, puis via brew, installer opencv, swig et pip :

    $ brew install opencv
    $ brew install swig
    $ brew install pip

Installer Numpy via PIP (de préférence avec sudo, si l'installation se fait dans ~/.local il peut y avoir des erreurs lors de la compilation du wrapper) :

    $ sudo python -m pip install numpy

Mettez à jour CMake si vous utilisez une version inférieure à 3.8, le plus simple est de passer par pip :

    $ python -m pip install --upgrade cmake

### Ajout du dossier /data
Il faut télécharger le dossier /data non présent sur notre dépôt et le placer dans /DocCreator (hébergé pour le moment par nos soins) :

    $ cd pdp-bfhl/DocCreator
    $ curl -L -o data.zip https://www.dropbox.com/sh/2tqo2z437zthn67/AAAVyrNm-ICarRMmvKMkslz7a?dl=1
    $ unzip data.zip
    $ rm -fr data.zip     #Le zip ne nous est plus utile
    $ cd ..


## Compilation
_Automatisée_ : Utilisez le script `compile-project.sh` présent à la racine du projet comme suit :

    $ ./compile-project.sh <MyInstallationPrefix>

_Manuelle_ : Depuis la racine du projet :

    $ cd DocCreator
    $ mkdir build
    $ cd build
    $ cmake .. -DCMAKE_INSTALL_PREFIX=<MyInstallationPrefix>
    $ make
    $ make install


## Utilisation : Degradations
Si vous n'en avez pas déjà un, installez un module Python chargeant les images au format numpy.array (cv2, imageio, matplotlib), on utilisera cv2 dans la suite de ce tutoriel :

    $ python -m pip install cv2

Déplacez vous vers le répertoire contenant les fichiers Python que l'on a créés à lors de la compilation, au choix :

    $ cd DocCreator/build/wrapper/python
    # OR
    $ cd <MyInstallationPrefix>/libpython

Vous pouvez utiliser vos propres images ou reprendre celles que nous prenons en exemple : https://www.dropbox.com/sh/59i34fhesb76p2k/AADAJpaRyTfFTtJdngJQmSeUa?dl=1

Pour les patterns de phantomCharactersDegradation, vous pouvez utiliser un chemin vers ceux du dossier `/data/Image/phantomPatterns` téléchargés précédemment ou télécharger le dossier phantomPatterns seul : https://www.dropbox.com/sh/c1nxzddyon4bqna/AAAggMLqanYgNGoNRJNZmAaPa?dl=1

> Ces images et ce dossier sont supposés placés au même niveau que les autres fichiers dans la suite de cet exemple.

Utilisez les fonctions dans un script Python ou directement depuis un terminal comme suit :
```python
import DocCreator
import cv2

img = cv2.imread("p00.png")
img2 = cv2.imread("p01.png")
pattern = cv2.imread("pattern1.png")

img3 = DocCreator.grayscaleChars(img, 5, 33, 33, 34)
img3 = DocCreator.grayscaleCharsByLevel(img3, 5)
img3 = DocCreator.shadowBinding(img3, 3, 500)
# Quand il y a beaucoup d'arguments on peut simplement les
# nommer pour pouvoir les passer dans n'importe quel ordre
img3 = DocCreator.holeDegradation(imgIn=img3, pattern=pattern, xOrigin=50, yOrigin=50, holetype=1, side=1, size=0, color=[220, 60, 60])
img3 = DocCreator.bleedThrough(img3, img2, 50)
img3 = DocCreator.phantomCharacter(img3, 2, "phantomPatterns/")
img3 = DocCreator.blurFilter(img3, 0, 11)

# Enregistrer le résultat
cv2.imwrite("p00_degraded.png", img3)
```

Notes :
- les kwargs, qui permettent de nommer les arguments, sont activés pour toutes les fonctions, si on décide de nommer les arguments il doivent tous être nommés pour cet appel de fonction.
- Toutes les fonctions opèrent sur des images définies au format RGB.


## Utilisation : Generation
Pour pouvoir appeler la génération, il faut lui fournir des paramètres sous la forme d'un dictionnaire Python, la fonction retourne une liste contenant toutes les images générées, les images sont de type np.array, voici un exemple d'utilisation avec tous les paramètres disponibles :
```python
import DocCreator

params = {
    'outputFolderPath' : './imgs',
    'backgroundList' : ['bg-1.png', 'montaigne-03.png'],
    'fontList' : ['JulesVerne', 'manto', 'BMTcomplete'], # From /data (no extension specified)
    'txtDirPath' : './inputTexts',  # Optionnel
    'bottomMarginMin' : 50,         # Optionnel
    'bottomMarginMax' : 50,         # Optionnel
    'leftMarginMin' : 50,           # Optionnel
    'leftMarginMax' : 50,           # Optionnel
    'rightMarginMin' : 50,          # Optionnel
    'rightMarginMax' : 50,          # Optionnel
    'topMarginMin' : 50,            # Optionnel
    'topMarginMax' : 50             # Optionnel
}

DocCreator.create(params)
```

Notes :
- txtDirPath : s'il n'est pas fourni, on utilise un texte Lorem Ipsum.
- marges : par défaut, elles ont toutes pour valeur 50.
- Les backgrounds et les fonts sont d'abord recherchés avec le chemin donné par l'utilisateur, et si ils n'existent pas, ils sont ensuite cherchés dans /data.
- Pour une font venant de /data, il ne faut pas préciser l'extension du fichier, si elle vient d'ailleurs, il faut le préciser.

## Etendre le wrapper en ajoutant de nouvelles fonctions (dégradations)
1. Dans `DocCreator/wrapper/Degradations.hpp` :
Depuis Python, on passe un numpy.array à nos fonctions, celui-ci est convertit par une typemap SWIG dans le fichier `Degradations.i` (l.16 : %apply...) en un pointeur uint8_t* de taille (nbLignes * nbColonnes * nbCanaux) et en trois arguments spécifiant la taille de chaque dimension que l'on pourra aisément convertir en cv::Mat ensuite, donc les premiers arguments de toute fonction souhaitant être wrappée commencent par :
`uint8_t *data, int nbRows, int nbCols, int nbChannels`, les deux suivants `uint8_t* imgOut, int imgOutLength` concernent l'image de retour, ses données et leur longueur (qui vaut donc `nbRows * nbCols * nbChannels` si la fonction est appelée correctement) on met ensuite les arguments de la dégradation, le type retourné est void car notre image retournée passe par les arguments (géré par la typemap issue de numpy.i dans Degradations.i l.20).

2. Dans `DocCreator/wrapper/Degradations.cpp` :
On commence toujours par convertir notre array `data` en une cv::Mat, on appelle la fonction de dégradation avec cette cv::Mat (ou une copie si elle est modifiée par la dégradation, sinon on risque de modifier la mémoire de l'image d'origine) et les autres arguments (ou si la dégradation travaille avec des QImage on utilise le Convertor pour passer d'un format à un autre), puis on copie les données de cette cv::Mat vers imgOut, la cv::Mat étant locale on ne peut pas se contenter de pointer vers les données de celle-ci, on ne peut pas non plus l'allouer complètement car nous n'aurons aucun moyen de la désallouer par la suite.

3. Dans `DocCreator/wrapper/DocCreator.py` :
On définit une courte fonction qui se chargera d'appeler la dégradation avec les arguments fournit et de reshape la sortie vers les dimensions d'origines avant de la retouner. Dans cette fonction on ignore volontairement de préciser l'argument imgOutLength qui reste caché à l'utilisateur, sa valeur est calculée en fonction des dimensions de l'image que l'on veut dégrader et est passé à la fonction de dégradation.
