#include "appconstants.h"
#include "core/configurationmanager.h"
#include <QDir>
#include <QDirIterator>

static bool checkDataDir(const QDir &dir) {
  //For now, we only check that it has a 'font' subdirectory
  return QDir(dir).cd(QStringLiteral("font"));
}

static QString findRelativeDataDir() {
  /*
    data/ may be:
    ../share/DocCreator/data installed on linux
    ../Resources/data installed on Mac
    ../../../data compiled on linux with build/ dir in sources
    ../../../<something>/data compiled on linux/mac with QtCreator
   */
  QDir dir(".");
  bool okup1 = dir.cdUp();
  if (okup1) {
    { //../share/DocCreator/data
      QDir dir1(dir);
      const bool ok1 = dir1.cd(QStringLiteral("share"));
      if (ok1) {
        const bool ok1b = dir1.cd(QStringLiteral("DocCreator"));
        if (ok1b) {
          const bool ok1c = dir1.cd(QStringLiteral("data"));
          if (ok1c) {
            if (checkDataDir(dir1)) {
              return dir1.absolutePath();
            }
          }
        }
      }
    }

    { //../Resources/data
      QDir dir1(dir);
      const bool ok1 = dir1.cd(QStringLiteral("Resources"));
      if (ok1) {
        const bool ok1b = dir1.cd(QStringLiteral("data"));
        if (ok1b) {
          if (checkDataDir(dir1)) {
            return dir1.absolutePath();
          }
        }
      }
    }

    { //../../../data
      QDir dir1(dir);
      const bool okup2 = dir1.cdUp();
      if (okup2) {
        const bool okup3 = dir1.cdUp();
        if (okup3) {
          const bool ok1 = dir1.cd(QStringLiteral("data"));
          if (ok1) {
            if (checkDataDir(dir1)) {
              return dir1.absolutePath();
            }
          }
        }
      }
    }

    { //../../../*/data
      QDir dir1(dir);
      const bool okup2 = dir1.cdUp();
      if (okup2) {
        const bool okup3 = dir1.cdUp();
        if (okup3) {
          QDirIterator it(dir1.absolutePath(), QDirIterator::Subdirectories);
          while (it.hasNext()) {
            it.next();
            QDir dir2(it.filePath());
            //std::cerr<<"search "<<dir2.absolutePath().toStdString()<<"\n";
            const bool ok2 = dir2.cd(QStringLiteral("data"));
            if (ok2) {
              if (checkDataDir(dir2)) {
                return dir2.absolutePath();
              }
            }
          }
        }
      }
    }
  }

  std::cerr << "Warning: findRelativeDataDir(): data directory not found !\n";

  return QString();
}

void initializeDataFolders() {
    QString dataDirStr = findRelativeDataDir();
    QDir dir(dataDirStr);

    Core::ConfigurationManager::set(
                        AppConfigMainGroup,
                        AppConfigFontFolderKey,
                        dir.absoluteFilePath(QStringLiteral("font")));
    Core::ConfigurationManager::set(
                        AppConfigMainGroup,
                        AppConfigBackgdFolderKey,
                        dir.absoluteFilePath(QStringLiteral("background")));

}

void initConfigManager() {
    Core::ConfigurationManager::initialize(AppConfigFile);
    initializeDataFolders();
}