# Informations

Ce projet est réalisé dans le cadre de l'UE **Projets de Programmation (PdP) 2018-2019** de Master 1 Informatique (Second semestre) de l'**Université de Bordeaux**. Les membres du groupe sont : Lilian Bellini, Guillaume Floret, Arthur Herman et Quentin Lamu.


# Installation et compilation du projet

Pour installer les dépendances requises pour le projet :
Sur Ubuntu/Debian, nous avons regroupés les commandes d'installations dans un script
```bash
sudo ./install-dependencies.sh
```

Pour les autres systèmes, il vous faut installer les dépendances suivantes :
- qt5
- opencv
- cmake (version 3.8 minimum)
- python
- python-dev (certains systèmes n'en ont pas besoin, ex : macOS)
- python-pip
- numpy (via pip)
- ply (via pip)
- opencv-python (via pip, requis pour exécuter nos tests)

Note : Si vous utilisez Ubuntu 16.04, CMake sera mis à jour durant l'installation des dépendances car le module SWIG de CMake demande une version de CMake supérieure à celle installée via apt sur Ubuntu 16.04 (3.5, et on veut 3.8+).

Pour la compilation le dossier /data de DocCreator est nécessaire. Pour alléger le dépôt, ce dossier n'est pas sur notre git, pour le récupérer, il faut donc faire :
```bash
cd DocCreator
curl -L -o data.zip https://www.dropbox.com/sh/2tqo2z437zthn67/AAAVyrNm-ICarRMmvKMkslz7a?dl=1
unzip data.zip
rm -fr data.zip
```

Pour la compilation et l'installation via script :
```bash
./compile-project.sh <MyInstallationPrefix>
```

Pour compiler à la main, sans installer :
```bash
cd DocCreator
mkdir build
cd build
cmake .. -DCMAKE_INSTALL_PREFIX=<MyInstallationPrefix>
make
#make install
```


## Utilisation

Notre projet se décompose en quatre parties princpales :
- Le wrapper Python : le fichier **README.md** (1ère partie) du dossier **/DocCreator/wrapper/python** donne toutes les informations nécessaires à son utilisation.
- L'interface en ligne de commande : le fichier **README.md** (2e partie) du dossier **/DocCreator/wrapper/python/cli** donne toutes les informations nécessaires à son utilisation.
- Le packaging _pip_ du projet : le fichier **README.md** du dossier **/DocCreatorPackage** donne toutes les informations nécessaires à son utilisation.


## Tests

Pour tester notre wrapper nous avons mis en place des séries de tests appelant les différentes dégradations en faisant varier leurs arguments, avec les fonctions Python puis C++ d'origine, puis nous comparons les images produites en sortie, pixel par pixel. Le **README.md** du dossier **/DocCreator/tests** contient toutes les informations nécessaires à leur exécution.


## Travail effectué

Comme notre projet est basé sur un logiciel existant, cette section a pour but de lister
les fichiers issus de notre travail, pour les distinguer des fichiers de DocCreator d'origine.

Les fichiers que nous avons créés :
- Tout le contenu de wrapper, excepté numpy.i, et le contenu de Generation_aux.cpp est constitué de fonctions copiées de software/DocCreator/src/DocCreator.cpp
- Tout le contenu de tests
- framework/src/RandomDocument/, la majorité du code vient de software/DocCreator/src/Degradations/ImageGenerationFromDirDialog.cpp
- DocCreatorPackage (depuis la racine)
- doc (depuis la racine)
- Tous les autres fichiers à la racine du dépôt

Les fichiers qui existaient mais que nous avons modifié pour intégrer notre travail ou
corriger des bugs :
- CMakeLists.txt
- framework/CMakeLists.txt
- framework/src/Degradations/GrayscaleCharsDegradationModel.cpp
- framework/src/Degradations/PhantomCharacter.cpp
- software/CMakeLists.txt
- software/DocCreator/CMakeLists.txt

Les fichiers où les appels à phantomCharacter ont été adaptés suite à nos modifications :
- framework/src/Degradations/PhantomCharacter.hpp
- software/DocCreator/src/Degradations/PhantomCharacterDialog.cpp
- software/DocCreator/src/Degradations/PhantomCharacterDialog.hpp
- software/DocCreator/src/GenerateDocument/Assistant.cpp
- software/DocCreator/src/GenerateDocument/Assistant.hpp

Les fichiers que nous avons déplacés (leur position actuelle) :
- framework/src/Utils/FontUtils.cpp
- framework/src/Utils/FontUtils.hpp
- framework/src/Utils/RandomElement.cpp
- framework/src/Utils/RandomElement.hpp
- framework/src/appconstants.h


## Autres dossiers

- **/doc** contient :
  - **/cahier_des_charges** contient notre cahier des besoins tel que rendu en février
  - **/memoire** contient notre mémoire de fin de projet (sera mis à jour rapidement)
